import { createRoot } from 'react-dom/client';
import { Suspense, lazy } from 'react';
import { preloadResource, addResourceHint } from './lib/performance';
import ErrorBoundary from './components/ui/error-boundary';
import { initPerformanceMonitoring, reportPerformance } from './lib/performance-monitoring';

// Preconnect to important domains
addResourceHint('https://fonts.googleapis.com', 'preconnect');
addResourceHint('https://fonts.gstatic.com', 'preconnect');
addResourceHint('https://cdn.gpteng.co', 'preconnect');

// Preload critical resources
preloadResource('/fonts/main-font.woff2', 'font');

// Initialize performance monitoring
initPerformanceMonitoring();

// Security header setup (would normally be on server, but simulating here)
const setupSecurityHeaders = () => {
  // Create a Content Security Policy meta tag
  const cspMeta = document.createElement('meta');
  cspMeta.httpEquiv = 'Content-Security-Policy';
  cspMeta.content = 
    "default-src 'self'; " +
    "script-src 'self' 'unsafe-inline' https://cdn.gpteng.co; " +
    "style-src 'self' 'unsafe-inline' https://fonts.googleapis.com; " +
    "img-src 'self' data: https:; " +
    "font-src 'self' https://fonts.gstatic.com; " +
    "connect-src 'self' https://lovable.dev";
  
  document.head.appendChild(cspMeta);
  
  // Create a CORS meta tag
  const corsMeta = document.createElement('meta');
  corsMeta.httpEquiv = 'Cross-Origin-Resource-Policy';
  corsMeta.content = 'same-site';
  document.head.appendChild(corsMeta);
  
  // Set up event listener for CSP violations
  document.addEventListener('securitypolicyviolation', (e) => {
    console.error('CSP violation:', e.blockedURI, 'violated', e.violatedDirective);
  });
  
  console.log('Security headers initialized');
};

// Initialize security headers
setupSecurityHeaders();

// Lazy load the App component to enable route-based code splitting
const App = lazy(() => import('./App.tsx'));

// Error fallback for the whole app
const AppErrorFallback = () => (
  <div className="flex items-center justify-center h-screen bg-white">
    <div className="text-center p-6 max-w-md">
      <div className="w-16 h-16 mx-auto mb-4">
        <svg xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24" stroke="currentColor" className="text-red-500">
          <path strokeLinecap="round" strokeLinejoin="round" strokeWidth={2} d="M12 9v2m0 4h.01m-6.938 4h13.856c1.54 0 2.502-1.667 1.732-3L13.732 4c-.77-1.333-2.694-1.333-3.464 0L3.34 16c-.77 1.333.192 3 1.732 3z" />
        </svg>
      </div>
      <h1 className="text-xl font-bold text-gray-900 mb-3">Something went wrong</h1>
      <p className="text-gray-600 mb-4">We apologize for the inconvenience. Please try refreshing the page.</p>
      <button
        className="px-4 py-2 bg-suku-red text-white rounded hover:bg-suku-red/90"
        onClick={() => window.location.reload()}
      >
        Refresh Page
      </button>
    </div>
  </div>
);

// Initialize the app with Suspense fallback and ErrorBoundary
createRoot(document.getElementById("root")!).render(
  <ErrorBoundary fallback={<AppErrorFallback />}>
    <Suspense fallback={
      <div className="flex items-center justify-center h-screen bg-white">
        <div className="text-center">
          <div className="w-16 h-16 border-4 border-suku-red border-t-transparent rounded-full animate-spin mx-auto"></div>
          <p className="mt-4 text-gray-600">Loading Suku Online...</p>
        </div>
      </div>
    }>
      <App />
    </Suspense>
  </ErrorBoundary>
);

// Report performance metrics
window.addEventListener('load', () => {
  // Use requestIdleCallback to avoid blocking the main thread
  if ('requestIdleCallback' in window) {
    (window as any).requestIdleCallback(() => {
      if ('performance' in window) {
        const navigation = performance.getEntriesByType('navigation')[0] as PerformanceNavigationTiming;
        if (navigation) {
          console.log(`Time to First Byte: ${Math.round(navigation.responseStart)}ms`);
          console.log(`DOM Content Loaded: ${Math.round(navigation.domContentLoadedEventEnd)}ms`);
          console.log(`Load Event: ${Math.round(navigation.loadEventEnd)}ms`);
        }
        
        // Report complete performance metrics after page has loaded
        setTimeout(() => {
          reportPerformance();
        }, 2000);
      }
    });
  }
});
